import argparse
import re

from PyWrtd import *

def cmd_sys_time(wrtd, args):
    print(wrtd.get_attr_string(wrtd.WRTD_GLOBAL_REP_CAP_ID,
                               wrtd.WRTD_ATTR_SYS_TIME))

def cmd_set_log(wrtd, args):
    print(args)

def cmd_clear_log(wrtd, args):
    print(args)

def cmd_list_rules(wrtd, args):
    print(args)

def cmd_add_rule(wrtd, args):
    print(args)

def cmd_set_rule(wrtd, args):
    print(args)

def cmd_remove_rule(wrtd, args):
    print(args)

def cmd_remove_all_rules(wrtd, args):
    print(args)

def cmd_enable_rule(wrtd, args):
    print(args)

def cmd_disable_rule(wrtd, args):
    print(args)

def cmd_disable_all_rules(wrtd, args):
    print(args)

def cmd_rule_info(wrtd, args):
    print(args)

def cmd_reset_rule_stats(wrtd, args):
    print(args)

def cmd_list_alarms(wrtd, args):
    print(args)

def cmd_add_alarm(wrtd, args):
    print(args)

def cmd_set_alarm(wrtd, args):
    print(args)

def cmd_remove_alarm(wrtd, args):
    print(args)

def cmd_remove_all_alarms(wrtd, args):
    print(args)

def cmd_enable_alarm(wrtd, args):
    print(args)

def cmd_disable_alarm(wrtd, args):
    print(args)

def cmd_disable_all_alarms(wrtd, args):
    print(args)

def cmd_alarm_info(wrtd, args):
    print(args)

def time_interval(string):
    m = re.match('\A([0-9]+)([pnums]?)\Z', string)
    if m == None:
        msg = "%r is not a valid time interval specification" % string
        raise argparse.ArgumentTypeError(msg)
    mult = {
        'p': 1e1,
        'n': 1e3,
        'u': 1e6,
        'm': 1e9,
        's': 1e12,
    }
    return int(m.group(1)) * int(mult[m.group(2)])

def time_interval_help():
    return """Default unit is ps, but an 'n','u','m' or 's' can be appended \
    to the value to set it to nano, micro, milli or full seconds"""

def main():

    parser = argparse.ArgumentParser(description='WRTD node configuration tool')
    parser.add_argument('-D', '--dev-id', dest='dev', required=True, type=int,
                        help='MockTurtle device ID (integer) to open')
    subparsers = parser.add_subparsers(title='Available commands',
                                       dest='command', metavar='<command>',
                                       help='(Use "<command> -h" to get more details)')
    subparsers.required = True;

    # sys-time
    cmd_parser = subparsers.add_parser('sys-time', help='Show current system time')
    cmd_parser.set_defaults(func=cmd_sys_time)

    # set-log
    cmd_parser = subparsers.add_parser('set-log', help='Enable/Disable logging')
    cmd_parser.add_argument('log', choices=['on', 'off'], help='Enable/Disable logging')
    cmd_parser.set_defaults(func=cmd_set_log)

    # clear-log
    cmd_parser = subparsers.add_parser('clear-log', help='Clear pending log entries')
    cmd_parser.set_defaults(func=cmd_clear_log)

    # list-rules
    cmd_parser = subparsers.add_parser('list-rules', help='List all defined Rules')
    cmd_parser.add_argument('-v', '--verbose', action='store_true',
                            help='Show details about each Rule')
    cmd_parser.set_defaults(func=cmd_list_rules)

    # add-rule
    cmd_parser = subparsers.add_parser('add-rule', help='Define a new Rule')
    cmd_parser.add_argument('name', help='The name of the new Rule')
    cmd_parser.set_defaults(func=cmd_add_rule)

    # set-rule
    cmd_parser = subparsers.add_parser('set-rule', help='Configure a Rule')
    cmd_parser.add_argument('name', help='The name of the Rule to configure')
    cmd_parser.add_argument('-d', '--delay', type=time_interval, default = 0,
                            help='Set the delay for this Rule. ' + time_interval_help())
    cmd_parser.add_argument('source', help='The source Event ID for this Rule.')
    cmd_parser.add_argument('destination', help='The destination Event ID for this Rule.')
    cmd_parser.set_defaults(func=cmd_set_rule)

    # remove-rule
    cmd_parser = subparsers.add_parser('remove-rule', help='Delete a Rule')
    cmd_parser.add_argument('name', help='The name of the Rule to delete')
    cmd_parser.set_defaults(func=cmd_remove_rule)

    # remove-all-rules
    cmd_parser = subparsers.add_parser('remove-all-rules', help='Delete all Rules')
    cmd_parser.set_defaults(func=cmd_remove_all_rules)

    # enable-rule
    cmd_parser = subparsers.add_parser('enable-rule', help='Enable a Rule')
    cmd_parser.add_argument('name', help='The name of the Rule to enable')
    cmd_parser.set_defaults(func=cmd_enable_rule)

    # disable-rule
    cmd_parser = subparsers.add_parser('disable-rule', help='Disable a Rule')
    cmd_parser.add_argument('name', help='The name of the Rule to disable')
    cmd_parser.set_defaults(func=cmd_disable_rule)

    # disable-all-rules
    cmd_parser = subparsers.add_parser('disable-all-rules', help='Disable all Rules')
    cmd_parser.set_defaults(func=cmd_remove_all_rules)

    # rule-info
    cmd_parser = subparsers.add_parser('rule-info', help='Display information about a Rule')
    cmd_parser.add_argument('name', help='The name of the Rule to display its information')
    cmd_parser.set_defaults(func=cmd_rule_info)

    # reset-rule-stats
    cmd_parser = subparsers.add_parser('reset-rule-stats', help='Reset all statistics of a Rule')
    cmd_parser.add_argument('name', help='The name of the Rule to reset its statistics')
    cmd_parser.set_defaults(func=cmd_reset_rule_stats)

    # list-alarms
    cmd_parser = subparsers.add_parser('list-alarms', help='List all defined Alarms')
    cmd_parser.add_argument('-v', '--verbose', action='store_true',
                            help='Show details about each Alarm')
    cmd_parser.set_defaults(func=cmd_list_alarms)


    # add-alarm
    cmd_parser = subparsers.add_parser('add-alarm', help='Define a new Alarm')
    cmd_parser.add_argument('name', help='The name of the new Alarm')
    cmd_parser.set_defaults(func=cmd_add_alarm)

    # set-alarm
    cmd_parser = subparsers.add_parser('set-alarm', help='Configure an Alarm')
    cmd_parser.add_argument('name', help='The name of the Alarm to configure')
    cmd_parser.add_argument('-d', '--delay', type=time_interval, default = 0,
                            help='Set the delay for this Alarm. ' + time_interval_help())
    cmd_parser.add_argument('-s', '--setup', type=time_interval, default = 0,
                            help='Set the setup time for this Alarm. ' + time_interval_help())
    cmd_parser.add_argument('-p', '--period', type=time_interval, default = 0,
                            help='Set the period for this Alarm. ' + time_interval_help())
    cmd_parser.add_argument('-c', '--count', type=int, default = 0,
                            help='Set the repeat count for this Alarm')
    cmd_parser.set_defaults(func=cmd_set_alarm)

    # remove-alarm
    cmd_parser = subparsers.add_parser('remove-alarm', help='Delete an Alarm')
    cmd_parser.add_argument('name', help='The name of the Alarm to delete')
    cmd_parser.set_defaults(func=cmd_remove_alarm)

    # remove-all-alarms
    cmd_parser = subparsers.add_parser('remove-all-alarms', help='Delete all Alarms')
    cmd_parser.set_defaults(func=cmd_remove_all_alarms)

    # enable-alarm
    cmd_parser = subparsers.add_parser('enable-alarm', help='Enable an Alarm')
    cmd_parser.add_argument('name', help='The name of the Alarm to enable')
    cmd_parser.set_defaults(func=cmd_enable_alarm)

    # disable-alarm
    cmd_parser = subparsers.add_parser('disable-alarm', help='Disable an Alarm')
    cmd_parser.add_argument('name', help='The name of the Alarm to disable')
    cmd_parser.set_defaults(func=cmd_disable_alarm)

    # disable-all-alarms
    cmd_parser = subparsers.add_parser('disable-all-alarms', help='Disable all Alarms')
    cmd_parser.set_defaults(func=cmd_remove_all_alarms)

    # alarm-info
    cmd_parser = subparsers.add_parser('alarm-info', help='Display information about an Alarm')
    cmd_parser.add_argument('name', help='The name of the Alarm to display its information')
    cmd_parser.set_defaults(func=cmd_alarm_info)

    args = parser.parse_args()

    dev = 'MT' + str(args.dev)

    wrtd = PyWrtd(dev)

    args.func(wrtd, args)

if __name__ == "__main__":
    main()
